
#ifdef LALIB_EXPORTS
#define LALIB_API __declspec(dllexport) 
#else
#define LALIB_API __declspec(dllimport) 
#endif

namespace LinearAlgebra
{
    //This class is exported from LAlib.DLL
    class LA
    {
    public:

		//global variables
		static LALIB_API int GPU_CC; //GPU compute capability

        	//openMP initialization
		static LALIB_API int Init_openMP( int max_cores ); 
        
		//CUDA initialization
		static LALIB_API int Init_CUDA( int iDev ); 

		// ********** Level-1 operations **********
		
		//copy vector
		static LALIB_API void CopyV( double *Vin, int nDim, double *Vout );

        //vector norm (sum)
		static LALIB_API double SumV( double *V, int nDim ); 
        
		//vector norm (sum(x^2))
		static LALIB_API double SumVV( double *V, int nDim ); 
		
		//vector norm (sqrt(sum(x^2)))
		static LALIB_API double SqrtVV( double *V, int nDim );
        
		//dot product of two vectors
		static LALIB_API double DotVV( double *V1, double *V2, int nDim ); 
		
		//Vector multiplied by scalar
		static LALIB_API void MultiplyVS( double *V, int nDim, double S, double *X );
		
		//Vector multiplied by scalar in openMP
		static LALIB_API void MultiplyVS1( double *V, int nDim, double S, double *X );

		//Vector multiplied by scalar on GPU
		static LALIB_API void MultiplyVS2( double *V, int nDim, double S, double *X );

		//Vector added to vector
		static LALIB_API void AddVV( double *V1, double *V2, int nDim, double *X );

		//Vector added to vector in openMP
		static LALIB_API void AddVV1( double *V1, double *V2, int nDim, double *X );

		//Vector added to vector on GPU
		static LALIB_API void AddVV2( double *V1, double *V2, int nDim, double *X );

		//Vector substracted from vector
		static LALIB_API void SubstractVV( double *V1, double *V2, int nDim, double *X );
		
		//Vector substracted from vector in openMP
		static LALIB_API void SubstractVV1( double *V1, double *V2, int nDim, double *X );

		//Vector substracted from vector on GPU
		static LALIB_API void SubstractVV2( double *V1, double *V2, int nDim, double *X );
		
		//Select elements of vector (indicated in vector with indices, nDim size)
		static LALIB_API void SelectV( double *V, int *VI, int nDim, double *X );

		//Find elements in vector ( Cond = -1 less than S, = 0 equal S, = 1 greater than S )
		static LALIB_API int FindV( double *V, int nDim, double S, int Cond, int *XI );

		//Sort (ascending) elements in vector
		static LALIB_API void SortV( double *V, int nDim, double *X );

		//Sort (ascending) elements in vector (return indices only)
		static LALIB_API void SortVI( double *V, int nDim, int *XI );

		//Zeroing vector
		static LALIB_API void ZeroV( double *V, int nDim );

		//Zeroing vector in openMP
		static LALIB_API void ZeroV1( double *V, int nDim );

		//Zeroing vector
		static LALIB_API void ZeroVI( int *VI, int nDim );

		//Zeroing vector in openMP
		static LALIB_API void ZeroVI1( int *VI, int nDim );
		
		//Unity vector
		static LALIB_API void OneVI( int *VI, int nDim );

		//Unity vector in openMP
		static LALIB_API void OneVI1( int *VI, int nDim );
		
		//Find maximum value in specified adresses
		static LALIB_API double MaxV( double *V, int iStart, int iStop, int step );
		
		//Find index of maximum value in specified adresses
		static LALIB_API int idMaxV( double *V, int iStart, int iStop, int step );

		//Multiply two vectors in table mode
		static LALIB_API void TableMultiplyVV( double *V1, double *V2, int nDim, double *X );
		
		//Multiply two vectors in table mode in openMP
		static LALIB_API void TableMultiplyVV1( double *V1, double *V2, int nDim, double *X );
		
		//Assemble two vectors with pattern
		static LALIB_API void AssembleVV( double *V1, int *VI, int nDim, double *V2, double *X );

		// ********** Level-2 operations **********

		//cross product of two vectors
		static LALIB_API void CrossVV( double *V1, int nRows, double *V2, int nCols, double *X );
		
		//cross product of two vectors in openMP
		static LALIB_API void CrossVV1( double *V1, int nRows, double *V2, int nCols, double *X );
		
		//cross product of two vectors on GPU
		static LALIB_API void CrossVV2( double *V1, int nRows, double *V2, int nCols, double *X );
		
		//Matrix multiplied by vector
		static LALIB_API void MultiplyMV( double *M, int nRowsM, double *V, int nRowsV, double *X );
		
		//Matrix multiplied by vector in openMP
		static LALIB_API void MultiplyMV1( double *M, int nRowsM, double *V, int nRowsV, double *X );
		
		//Matrix multiplied by vector on GPU
		static LALIB_API void MultiplyMV2( double *M, int nRowsM, double *V, int nRowsV, double *X );
		
		//Vector multiplied by matrix
		static LALIB_API void MultiplyVM( double *V, int nColsV, double *M, int nColsM, double *X );
		
		//Vector multiplied by matrix in openMP
		static LALIB_API void MultiplyVM1( double *V, int nColsV, double *M, int nColsM, double *X );

		//Vector multiplied by matrix on GPU
		static LALIB_API void MultiplyVM2( double *V, int nColsV, double *M, int nColsM, double *X );
		
		//Matrix multiplied by scalar
		static LALIB_API void MultiplyMS( double *M, int nRows, int nCols, double S, double *X );
		
		//Matrix multiplied by scalar in openMP
		static LALIB_API void MultiplyMS1( double *M, int nRows, int nCols, double S, double *X );

		//Matrix multiplied by scalar on GPU
		static LALIB_API void MultiplyMS2( double *M, int nRows, int nCols, double S, double *X );
		
		//Matrix added to matrix
		static LALIB_API void AddMM( double *M1, double *M2, int nRows, int nCols, double *X );
		
		//Matrix added to matrix in openMP
		static LALIB_API void AddMM1( double *M1, double *M2, int nRows, int nCols, double *X );

		//Matrix added to matrix on GPU
		static LALIB_API void AddMM2( double *M1, double *M2, int nRows, int nCols, double *X );		
		
		//Matrix substracted from matrix
		static LALIB_API void SubstractMM( double *M1, double *M2, int nRows, int nCols, double *X );
		
		//Matrix substracted from matrix in openMP
		static LALIB_API void SubstractMM1( double *M1, double *M2, int nRows, int nCols, double *X );

		//Matrix substracted from matrix on GPU
		static LALIB_API void SubstractMM2( double *M1, double *M2, int nRows, int nCols, double *X );
		
		//Matrix transposition
		static LALIB_API void TransposeM( double *M, int nRows, int nCols, double *X );
		
		//Matrix transposition in openMP
		static LALIB_API void TransposeM1( double *M, int nRows, int nCols, double *X );
		
		//Square sub-matrix selection 
		static LALIB_API void SelectMsq( double *M, int nDim, int *VI, double *X );
		
		//Square sub-matrix aggregation 
		static LALIB_API void AggregateMMsq( double *m, int mDim, int *VI, int vDim, double *M, int MDim );

		// ********** Level-3 operations **********

		//Solves Linear System of Equations
        static LALIB_API void SolveLSEauto( double *A, double *B, double *X, int nDim, int n1, int n2 ); 
        
		//standard Gauss (cpu)
		static LALIB_API void SolveLSE( double *A, double *B, double *X, int nDim );
		
		//Gauss in openMP
        static LALIB_API void SolveLSE1( double *A, double *B, double *X, int nDim ); 

		//Gauss-Jordan (gpu)
        static LALIB_API void SolveLSE2( double *A, double *B, double *X, int nDim );
		
		//Gauss (gpu/cpu)
        static LALIB_API void SolveLSE3( double *A, double *B, double *X, int nDim ); 
		
		//Matrix multiplied by matrix
        static LALIB_API void MultiplyMM( double *M1, int nRows1, int nCols, double *M2, int nCols2, double *X ); 
		
		//Matrix multiplied by matrix in openMP
        static LALIB_API void MultiplyMM1( double *M1, int nRows1, int nCols, double *M2, int nCols2, double *X ); 

		//Matrix multiplied by matrix on GPU
        static LALIB_API void MultiplyMM2( double *M1, int nRows1, int nCols, double *M2, int nCols2, double *X ); 
		
		//Inversion of square matrix 
		static LALIB_API void InverseMsq( double *M, double *iM, int nDim );
		
		//Determinant of square matrix (Gauss)		
		static LALIB_API double DeterminantMsq( double *M, int nDim );
		
		//Eigenvalues of unsymmetric real square matrix
		static LALIB_API void EigenUnSymmSystemMsq( double *M, int nDim, double *eV_r, double *eV_i );
		
		//Eigenvalues and eigenvectors of symmetric real square matrix - Jacobi
		static LALIB_API void EigenSymmSystemMsqJ( double *M, int nDim, double *eV, double *eM );
		
		//Eigenvalues and eigenvectors of symmetric real square matrix - Householder + QL
		static LALIB_API void EigenSymmSystemMsqH( double *M, int nDim, double *eV, double *eM );
	
		// ********** Random number generator **********
		
		//Initializes random number generator with a seed
		static LALIB_API void Init_MT( void );

		//Initialize by an array with array-length 
		//init_key is the array for initializing keys
		//key_length is its length
		//slight change for C++, 2004/2/26
		static LALIB_API void Init_MT_by_array( void );

		//Generates a random number on [0,0xffffffff]-interval 
		static LALIB_API unsigned long MT_u32( void );
	
		//Generates a random number on [0,0x7fffffff]-interval 
		static LALIB_API long MT_i31( void );

		//Generates a random number on [0,1]-real-interval 
		static LALIB_API double MT_d64( void );

		//Generates a random number on (0,1]-real-interval 
		static LALIB_API double MT_d64_0( void );

		//Generates a random number on [0,1)-real-interval 
		static LALIB_API double MT_d64_1( void );

		//Generates a random number on (0,1)-real-interval 
		static LALIB_API double MT_d64_01( void );

		//Generates a random number on [0,1) with 53-bit resolution
		static LALIB_API double MT_d53_1( void );
		
		//Generates a random number with normal distribution
		static LALIB_API double MT_Gauss64( double mu, double sigma );
		
		//Generates an array of random numbers with normal distribution
		static LALIB_API void MT_Gauss64_Array( double mu, double sigma, int n, double *A );
    };
}

namespace ConsoleFunctions
{
    //This class is exported from LAlib.DLL
    class CRT
    {
    public:
        //clearing screen
		static LALIB_API void clrscr( void ); 
        
		//moves cursor to point (x,y)
		static LALIB_API void gotoxy( int x, int y ); 
        
		//white text on black background - the default settings
		static LALIB_API void SetTextDefault( void ); 
		
		//reversed settings - black text on white background
		static LALIB_API void SetTextReverse( void );
		
		//set text and background colors
		static LALIB_API void SetTextColor( int fColor, int bColor );
		
		//clears line from x1 to x2 in y
		static LALIB_API void clrline( int x1, int x2, int y );
		
		//reads pressed key		
		static LALIB_API char keypressed( void );

		//exit function
		static LALIB_API void Press_and_Exit( int res );
		
		//exit function
		static LALIB_API void Enter_and_Exit( int res );
	};
	
	//This class is exported from LAlib.DLL
    class RS232
    {
    public:
        //sending a byte through RS232
		static LALIB_API int SendByte( unsigned char dat, unsigned char port, unsigned int baudRate ); 
		//sending one char through RS232
		static LALIB_API int SendChar( unsigned char dat, unsigned char port, unsigned int baudRate );
		//sending a byte to Wall-E ver.1 built with C51 89C4051 microprocessor
		static LALIB_API int Wall_Ev1( int step, int nsteps, int blink, unsigned char port );
		//sending a byte to Wall-E ver.2 built with ARM LPC2106 microprocessor
		static LALIB_API int Wall_Ev2( int step, int nsteps, int blink, unsigned char port );	 
	};
	
}

